uab.modules.FrameGroup = uab:NewModule("FrameGroup", "AceModuleCore-2.0")
uab.modules.FrameGroup:SetModuleMixins("AceDebug-2.0", "AceModuleCore-2.0")

uab.modules.FrameGroup.core = uab

uabFrameGroup = uab.modules.FrameGroup
uabFrameGroup.core = uab
uabFrameGroup.framegroups = {}

function uabFrameGroup:OnInitialize()
	self.super.OnInitialize(self)
end

function uabFrameGroup:OnEnable()
	self.super.OnEnable(self)
end

function uabFrameGroup:NewActivator(activator)
	for name, module in self:IterateModules() do
		module:NewActivator(activator)
	end
end

function uabFrameGroup:GetFrameGroup(frame)
	for name, module in self:IterateModules() do
        if module:IsFrameGroup(frame) then
			if module:IsEnabled() then
                return module
            end
		end
	end
end

function uabFrameGroup:HaveFrameControl(frame, activator)
	local framegroup = self:GetFrameGroup(frame)

	if framegroup then
	    return framegroup:HaveFrameControl(frame, activator)
	end
end

function uabFrameGroup:NextFrameControl(frame, activator)
	local framegroup = self:GetFrameGroup(frame)

	if framegroup then
        return framegroup:NextFrameControl(frame, activator)
	end
end

--
--function uabFrameGroup:RefreshFrameGroupList()
--	local order = 10
--
--	local names = {}
--	local moduleOptions = {}
--    
--	for name, module in self:IterateModules() do
--        local options = module:CreateFrameGroupOptions()
--        
--	    table.insert(names, name)
--	    moduleOptions[name] = options
--	end
--
--	table.sort(names)
--
--	self.core.options.args.lookandfeel.args.framegroups.args = {}
--
--    if # names > 0 then    
--        order = 100
--        for idx = 1, # names, 1 do
--            for name, args in pairs(moduleOptions[names[idx]]) do
--                self.core.options.args.lookandfeel.args.framegroups.args[name] = args
--                self.core.options.args.lookandfeel.args.framegroups.args[name].order = order
--                order = order + 1
--            end
--        end
--    end
--end
--

--[[
FrameGroup module prototype

When implementing a FrameGroup module:
 - variables to set
 	- self.defaultDB to define your own default settings
 	    be sure to include the following settings as they are used to control
 	    the matching of frames, and the scaling and positioning of uab buttons
 	    in relation to these frames:
 	    - disabled
 	    - expression (used to match frames to this frame group)
 	    - usedefaultscale
		- usedefaultanchor
		- point, relpoint, x, y for default positioning, if not using the
		    uab defaults that the user defines.
 	- self.unitgroup to define the unitgroup with which this frame group is
	 	associated
 - OnInitialize functionality to implement
    - set self.defaultDB here if needed
    - call super.OnInitialize(self)
    - set any other variables you may need and perform any additional first-time
		initialization
 - Methods to implement
	- OnEnable (optional)
	- OnDisable (optional)
    - NewActivator
        Only override this if you need to support more than one frame per unit
		group for each activator, or if you need to support more than one unit
		group
	- HaveFrameControl(frame, activator)
        Only override this if you need to support more than one frame per unit
		group for each activator, or if you need to support more than one unit
		group
	- NextFrameControl(frame, activator)
        Only override this if you need to support more than one frame per unit
		group for each activator, or if you need to support more than one unit
		group
	- CreateFrameGroupOptions(name)
	    Only override this if you need to support more than one unit group, or
	    need to provide additional configuration options.  Be sure to call
	    CreateFrameGroupOption(name) to create the options for the frame groups
	    that you need.
	- OptionsForFrame(frame)
	    Override this if you need to support more than one unit group, since
	    the default implementation assumes a single expression to match the
	    frame with a frame group.
--]]

uabFrameGroup.modulePrototype.core = uabFrameGroup
uabFrameGroup.modulePrototype.controls = {}
uabFrameGroup.modulePrototype.frameGroupDefaultDB = {
    disabled = nil,
    expression = nil,
    usedefaultscale = true,
    usedefaultanchor = true,
}

function uabFrameGroup.modulePrototype:OnInitialize()
	if not self.db then
		self.core.core:RegisterDefaults(self.name, "profile", self.defaultDB or self.frameGroupDefaultDB)
		self.db = self.core.core:AcquireDBNamespace(self.name)
	end

    self.debugging = self.db.profile.debug

    self.core.core.options.args.lookandfeel.args.framegroups.args[self.name] = self:CreateFrameGroupOptions()
end

function uabFrameGroup.modulePrototype:IsEnabled()
    return not self.db.profile.disabled
end

function uabFrameGroup.modulePrototype:IsFrameGroup(frame)
    if not self.db.profile.disabled then
        local options = self:OptionsForFrame(frame)
        
        if options and not options.disabled then
            return true
        end
    end
end

function uabFrameGroup.modulePrototype:NewActivator(activator)
	self.controls[activator] = activator:NewUnitGroupControl(self.unitgroup)
end

function uabFrameGroup.modulePrototype:HaveFrameControl(frame, activator)
	local options = self:OptionsForFrame(frame)

	if options and not self.db.profile.disabled and not options.disabled then
	    return self.controls[activator]
	end
end

function uabFrameGroup.modulePrototype:NextFrameControl(frame, activator)
	local options = self:OptionsForFrame(frame)
	
	if options and not self.db.profile.disabled and not options.disabled then
	    local control = self.controls[activator]
	    
	    self.controls[activator] = nil
	    return control
	end
end

function uabFrameGroup.modulePrototype:CreateFrameGroupOptions()
    return self:CreateFrameGroupOption(self.name)
end

function uabFrameGroup.modulePrototype:OptionsForFrame(frame)
    if self.db.profile.disabled then return end
    
    local framename = frame:GetName()

    if self:MatchFrame(frame, self.db.profile) then
        return self.db.profile
    end
end

function uabFrameGroup.modulePrototype:MatchFrame(frame, defn)
    return string.find(frame:GetName(), defn.expression)
end

function uabFrameGroup.modulePrototype:CreateFrameGroupOption(profile, name, displayname)
    local options
    local me = self

    if not name then
        name = self.name
    end

    if not displayname then
        displayname = name
    end

    options = {
        type = 'group',
        name = name,
        desc = "Use, anchor, scale, and match expression for frames in the "..displayname.." frame group",
        args = {
            enabled = {
                type = 'toggle',
                name = "Enabled",
                desc = "When checked, enable the use of UAB action sets for frames in the "..displayname.." frame group",
                order = 1,
                get = function()
                    return not profile.disabled
                end,
                set = function(v)
                    profile.disabled = not v
                    me.core.core.main:FrameGroupsChanged()
                end,
            },
            
            expression = {
                type = 'text',
                name = "Expression",
                usage = "<expression>",
                desc = "Define the match expression for frames in this frame group.  Any frame that has a name that matches this expression will use the anchor and scale settings defined for this frame group",
                order = 2,
                get = function()
                    return profile.expression
                end,
                set = function(exp)
                    profile.expression = exp
                    me.core.core.main:SetFrameGroupExpression(name, exp)
                end,
            },
            
            spacing3 = {
                name = " ",
                type = 'header',
                order = 5,
            },
        
            usedefaultscale = {
                type = 'toggle',
                name = "Default scale",
                desc = "Frames in the "..displayname.." frame group should use the default scale value",
                order = 10,
                get = function()
                    return profile.usedefaultscale
                end,
                set = function(v)
                    profile.usedefaultscale = v
                    if not v then
                        if not profile.scale then
                            profile.scale = me.core.core.db.profile.scale.default.value
                        end
                    end
                
                    self.core.core.main:ScaleChanged()
                end,
            },
            usedefaultanchor = {
                type = 'toggle',
                name = "Default anchor",
                desc = "Frames in the "..displayname.." frame group should use the default action button anchor information",
                order = 20,
                get = function()
                    return profile.usedefaultanchor
                end,
                set = function(v)
                    profile.usedefaultanchor = v
                    if not v then
                        if not profile.point then
                            profile.point = me.core.core.db.profile.position.default.point
                        end
                        
                        if not profile.relpoint then
                            profile.relpoint = me.core.core.db.profile.position.default.relpoint
                        end
                        
                        if not profile.x then
                            profile.x = me.core.core.db.profile.position.default.x
                        end
                        
                        if not profile.y then
                            profile.y = me.core.core.db.profile.position.default.y
                        end
                    end
                    
                    me.core.core.main:PositionChanged()
                end,
            },
            
            spacing3 = {
                name = " ",
                type = 'header',
                order = 30,
            },
        
            scale = {
                type = 'range',
                name = "Scale",
                desc = "Scale for frames in the "..displayname.." frame group",
                disabled = function() return profile.usedefaultscale end,
                order = 40,
                min = 0.05,
                max = 2.00,
                step = 0.05,
                isPercent = true,
                get = function()
                    return profile.scale or me.core.core.db.profile.scale.default.value
                end,
                set = function(value)
                    profile.scale = value
                    me.core.core.main:ScaleChanged()
                end,
            },
            point = {
                type = 'text',
                name = "Point",
                desc = "Select the point on the cancel button that positioning in based from for frames in the "..displayname.." frame group",
                disabled = function() return profile.usedefaultanchor end,
                order = 50,
                get = function()
                    return profile.point or me.core.core.db.profile.position.default.point or "CENTER"
                end,
                set = function(pt)
                    profile.point = pt
                    me.core.core.main:PositionChanged()
                end,
                validate = {
                    "BOTTOM",
                    "BOTTOMLEFT",
                    "BOTTOMRIGHT",
                    "CENTER",
                    "LEFT",
                    "RIGHT",
                    "TOP",
                    "TOPLEFT",
                    "TOPRIGHT",
                },
            },
            relativepoint = {
                type = 'text',
                name = "Relative Point",
                desc = "Select the point the buttons are relative to for the frames in the "..displayname.." frame group",
                disabled = function() return profile.usedefaultanchor end,
                order = 60,
                get = function()
                    return profile.relpoint or me.core.core.db.profile.position.default.relpoint or "cursor"
                end,
                set = function(pt)
                    profile.relpoint = pt
                    me.core.core.main:PositionChanged()
                end,
                validate = {
                    "cursor",
                    "screen",
                    "BOTTOM",
                    "BOTTOMLEFT",
                    "BOTTOMRIGHT",
                    "CENTER",
                    "LEFT",
                    "RIGHT",
                    "TOP",
                    "TOPLEFT",
                    "TOPRIGHT",
                },
            },
            horizontal = {
                type = 'range',
                name = "Horizontal",
                desc = "Horizontal offset (positive is right, negative is left, 0 is over cursor) for frames in the "..displayname.." frame group",
                disabled = function() return profile.usedefaultanchor end,
                order = 70,
                min = -100,
                max = 100,
                step = 1,
                isPercent = false,
                get = function()
                    return profile.x or me.core.core.db.profile.position.default.x
                end,
                set = function(value)
                    profile.x = value
                    me.core.core.main:PositionChanged()
                end,
            },
            vertical = {
                type = 'range',
                name = "Vertical",
                desc = "Vertical offset (positive is up, negative is down, 0 is over cursor) for frames in the "..displayname.." frame group",
                disabled = function() return profile.usedefaultanchor end,
                order = 80,
                min = -100,
                max = 100,
                step = 1,
                isPercent = false,
                get = function()
                    return profile.y or me.core.core.db.profile.position.default.y
                end,
                set = function(value)
                    profile.y = value
                    me.core.core.main:PositionChanged()
                end,
            },
        },
    }
    
    return options
end

