uab.modules.GridFrameGroup = uabFrameGroup:NewModule("Grid")
uab.modules.GridFrameGroup.controls = {}
uab.modules.GridFrameGroup.defaultDB = {
    framegroups = {
        ["Player"] = {
            expression = "^GridLayoutPartyPlayer$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "player",
        },
        ["Party"] = {
            expression = "^GridLayoutPartyHeaderUnitButton.+",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
        ["Raid"] = {
            expression = "^GridLayoutHeader.+UnitButton.+",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
    },
}

--[[
uab.modules.GridFrameGroup.framesPerUnit.player = 1
uab.modules.GridFrameGroup.framesPerUnit.partyraid = 1
--]]

function uab.modules.GridFrameGroup:OnInitialize()
    self.super.OnInitialize(self)
end

function uab.modules.GridFrameGroup:OnEnable()
end

function uab.modules.GridFrameGroup:OnDisable()
end

function uab.modules.GridFrameGroup:NewActivator(activator)
	self.controls[activator] = {}
    self:AllocateControlsForUnitGroup(activator, "player")
    self:AllocateControlsForUnitGroup(activator, "partyraid")
end

function uab.modules.GridFrameGroup:AllocateControlsForUnitGroup(activator, groupname)
    self.controls[activator][groupname] = {}
	for idx = 1, uab.constants.groupInfo[groupname].unitcount do
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
    end
end

function uab.modules.GridFrameGroup:HaveFrameControl(frame, activator)
	if self.db.profile.disabled then return end

	local options = self:OptionsForFrame(frame)
	
	if not options or options.disabled then return end

	if self.controls[activator] and self.controls[activator][options.unitgroup] then
		return # self.controls[activator][options.unitgroup] > 0
	end
end

function uab.modules.GridFrameGroup:NextFrameControl(frame, activator)
	if self.db.profile.disabled then return end

	local options = self:OptionsForFrame(frame)

	if not options or options.disabled then return end
	
    if self.controls[activator] and self.controls[activator][options.unitgroup]
        and # self.controls[activator][options.unitgroup] > 0 then
		control = table.remove(self.controls[activator][options.unitgroup])
	end

	return control
end

function uab.modules.GridFrameGroup:OptionsForFrame(frame)
    if self.db.profile.disabled then return end
    
    for name, defn in pairs(self.db.profile.framegroups) do
        if self:MatchFrame(frame, defn) then
            return defn
        end
    end
end

function uab.modules.GridFrameGroup:CreateFrameGroupOptions()
    local args = {}

    args = {
        type = 'group',
        name = "Grid",
        desc = "Use, anchor, scale, and match expression for the Grid frames",
        args = {
            enabled = {
                type = 'toggle',
                name = "Enabled",
                desc = "When checked, enable the use of UAB action sets for Grid frames",
                order = 1,
                get = function()
                    return not self.db.profile.disabled
                end,
                set = function(v)
                    self.db.profile.disabled = not v
                    self.core.core.main:FrameGroupsChanged()
                end,
            },
            
            spacing3 = {
                name = " ",
                type = 'header',
                order = 5,
            },
        
        },
    }
    
    args.args["player"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Player"], "Player", "Grid Player")
    args.args["player"].disabled = function() return self.db.profile.disabled end
    args.args["player"].order = 10
    
    args.args["party"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Party"], "Party", "Grid Party")
    args.args["party"].disabled = function() return self.db.profile.disabled end
    args.args["party"].order = 30
    
    args.args["raid"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Raid"], "Raid", "Grid Raid")
    args.args["raid"].disabled = function() return self.db.profile.disabled end
    args.args["raid"].order = 40
    
    return args
end

